import org.ejml.simple.SimpleMatrix;

import java.util.function.Function;

public class ActivationLayer extends Layer {
    Function<SimpleMatrix, SimpleMatrix> activation;
    Function<SimpleMatrix, SimpleMatrix> activationPrime;

    public ActivationLayer() {}

    public ActivationLayer(Function<SimpleMatrix, SimpleMatrix> activation, Function<SimpleMatrix, SimpleMatrix> activationPrime) {
        this.activation = activation;
        this.activationPrime = activationPrime;
    }

    @Override
    public SimpleMatrix forwardPropagation(SimpleMatrix input) {
        this.input = input;
        this.output = tanh(input);
        return this.output;
    }

    @Override
    public SimpleMatrix backwardPropagation(SimpleMatrix outputError, Double learningRate) {
        return tanhPrime(this.input).elementMult(outputError);
    }

    private SimpleMatrix tanh(SimpleMatrix A) {
        SimpleMatrix B = new SimpleMatrix(A);
        for (int i = 0; i < A.getNumElements(); i++) {
            B.set(i, Math.tanh(A.get(i)));
        }
        return B;
    }

    private SimpleMatrix tanhPrime(SimpleMatrix A) {
        SimpleMatrix B = new SimpleMatrix(A);
        for (int i = 0; i < A.getNumElements(); i++) {
            B.set(i, 1 - Math.pow(Math.tanh(A.get(i)), 2));
        }
        return B;
    }
}
